
#include "SDL.h"

#include <stdio.h>
#include <stdlib.h>

#include "video.h"
#include "audio.h"
#include "timer.h"
#include "event.h"
#include "render.h"
#include "game.h"

#include "common.h"

#include "main.h"

int main(void) {
  printf("DIM3NSION v"GAME_VERSION_STRING"\nStarting...\n\n");

  atexit(Main_DeInit);

  if(Main_Init()) {
    printf("Main_Init failed, shutting down...\n\n");
    return 1;
  }

  while(1) {
    if(Timer_Frame()) break;
    if(Event_Frame()) break;
    if(Render_Frame()) break;
    if(Audio_Frame()) break;
    if(Video_Frame()) break;
    if(Game_Frame()) break;

    Game_AfterFrame();
    Video_AfterFrame();
    Audio_AfterFrame();
    Render_AfterFrame();
    Event_AfterFrame();
    Timer_AfterFrame();
  }

  return 0;
}

int Main_Init(void) {
  printf("Init\n");

  if(SDL_Init(SDL_INIT_EVERYTHING) != 0) {
    printf("SDL_Init Failed: %s\n", SDL_GetError());
    return 1;
  }

  if(Common_Init()) return 1;
  if(Video_Init()) return 1;
  if(Audio_Init()) return 1;
  if(Timer_Init()) return 1;
  if(Render_Init()) return 1;
  if(Game_Init()) return 1;

  printf("\n");

  return 0;
}

void Main_DeInit(void) {
  printf("Shutdown\n");

  Game_DeInit();
  Render_DeInit();
  Audio_DeInit();
  Video_DeInit();

  SDL_Quit();
}
