
#include <SDL.h>
#include <GL/gl.h>

#include <stdio.h>
#include <string.h>

#include "common.h"

#include "model.h"

int Model_Init (void) {
  return 0;
}

int Model_LoadFromFile(const char *path, int id) {
  union {
    unsigned char  c[4];
    short s;
    int   i;
    float f;
  } bt;

  char header[8];

  int numframes, frame, numtris, tri;

  SDL_RWops *rw = SDL_RWFromFile(path, "rb");

  // check header
  SDL_RWread(rw, header, sizeof(char) * 6, 1);
  if(strncmp(header, MODEL_HEADER, 6)) {
    printf("Incorrect model header: expected `"MODEL_HEADER"', got `%s'\n",\
	   header);
    return 1;
  }

  // check numframes and numtris
  SDL_RWread(rw, bt.c, sizeof(int), 1); numframes = BigInt(bt.i);
  SDL_RWread(rw, bt.c, sizeof(int), 1); numtris = BigInt(bt.i);

  printf("Loading model `%s', F:%d T:%d\n", path, numframes, numtris);

  float x, y, z, nx, ny, nz, r, g, b;

  for(frame = 0; frame < numframes; frame++) {
    Model_DisplayLists[id][frame] = glGenLists(1);
    glNewList(Model_DisplayLists[id][frame], GL_COMPILE);
    
    glBegin(GL_TRIANGLES);
    
    for(tri = 0; tri < numtris; tri++) {
      SDL_RWread(rw, bt.c, sizeof(float), 1); r = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); g = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); b = BigFloat(bt.f);
      glColor3f(r, g, b);

      SDL_RWread(rw, bt.c, sizeof(float), 1); nx = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); ny = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); nz = BigFloat(bt.f);
      glNormal3f(nx, ny, nz);
      
      SDL_RWread(rw, bt.c, sizeof(float), 1); x = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); y = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); z = BigFloat(bt.f);
      glVertex3f(x, y, z);

      SDL_RWread(rw, bt.c, sizeof(float), 1); x = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); y = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); z = BigFloat(bt.f);
      glVertex3f(x, y, z);

      SDL_RWread(rw, bt.c, sizeof(float), 1); x = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); y = BigFloat(bt.f);
      SDL_RWread(rw, bt.c, sizeof(float), 1); z = BigFloat(bt.f);
      glVertex3f(x, y, z);
    }
    
    glEnd();
    
    glEndList();
  }

  SDL_RWclose(rw);

  return 0;
}
